%% The local SVD-FMM
% http://dx.doi.org/10.17864/1947.329
% Please see the README file for more information including licence.

function [q_near,q_inter] = algorithm_core(invR_near,d,B,S,V,box,near,iinter)
    m=size(box,1);
    d_box=cell(m,1);
    d_near=cell(m,1);
    for ibox=1:m
        d_box{ibox}=d(box{ibox});
        d_near{ibox}=d(near{ibox});
    end

    % Multipole expansions 
    Phi=cellfun(@multipole,d_box,V,'UniformOutput',false); 
    
    % Local expansions 
    Psi=cell(m,1);
    for ibox=1:m
        n=size(iinter{ibox},2);
        nbox=iinter{ibox}(1);
        Psi{ibox}=B{ibox}{nbox}*Phi{nbox};
        for i=2:n
            nbox=iinter{ibox}(i);
            Psi{ibox}=Psi{ibox}+B{ibox}{nbox}*Phi{nbox};
        end
    end
    
    % Interaction-field contribution
    q_inter=cellfun(@inter_field,V,S,Psi,'UniformOutput',false); 
    
    % Near-field contribution -----------------------
    q_near=cellfun(@near_field,invR_near,d_near,'UniformOutput',false); 
end

function Phi=multipole(d_box,V)
    Phi=V'*d_box;
end

function q_near=near_field(invR_near,d_near)
    q_near=invR_near*d_near;
end

function q_inter=inter_field(V,S,Psi)
    q_inter=V*S*Psi;
end

%% Notes
% The outputs "q_near" and "q_inter" are two cell arrays conating the results
% of near-field and interaction-field calculations respectively.
% Each cell element contains the results for the observations within a box. 