%% The local SVD-FMM
% http://dx.doi.org/10.17864/1947.329
% Please see the README file for more information including licence.

function  [U,S,V,B]  = algorithm_decomposition(Ko,box,inter,iinter,p)
    m=size(box,1);
    U=cell(m,1);
    S=cell(m,1);   
    V=cell(m,1);
    
    for ibox=1:m
        [U{ibox},S{ibox},V{ibox}]=svds(Ko{ibox},p);
    end
    
    % Compute the M2L translation operator
    B=cell(m,1);

    for ibox=1:m
        n=size(iinter{ibox},2);
        B{ibox}=cell(n,1);
        for i=1:n
            source_box=iinter{ibox}(i);
            indices=find_indices_interaction_list(box,inter,source_box,ibox);
            B{ibox}{source_box}=ifo_core(U{ibox}(indices,:),V{source_box},p);
        end
    end   
end

function indices=find_indices_interaction_list(box,inter,source_box,target_box)
    len=size(box{source_box},1);
    indices=zeros(1,len);
    for i=1:len
        indices(i)=find(inter{target_box}==box{source_box}(i));
    end
end

function B=ifo_core(U_target,V_source,p)
    B=zeros(p,p);
    for im=1:p
        for ik=1:p
            B(im,ik)=sum(U_target(:,im).*V_source(:,ik));
        end
    end
end

%% Notes
% The outputs are cell arrays.
% Each cell element contains the results for one box.
% U, S and V are left singular vectors, singular values and right singular
% vectors respectively.
% B is the multipole-to-local(M2L)translation operator. 