%% The local SVD-FMM
% http://dx.doi.org/10.17864/1947.329
% Please see the README file for more information including licence.

function [iinter,near,inter,invR_svd,invR_near]=algorithm_fields(box,kernel,r1,r2,modification)
    m=size(box,1);
    A=Zorder(sqrt(m));

    if modification==0
        % Inidces of observations
        near=cell(m,1);
        inter=cell(m,1);
        % Inidces of boxes
        inear=cell(m,1);
        iinter=cell(m,1);       

        ibox1=get(A,1,1);
        ibox2=get(A,r1,r2);
        for i=1:size(A,1)
            for j=1:size(A,2)
                inear{A(i,j)}=ibox1{i,j};
                iinter{A(i,j)}=setdiff(ibox2{i,j},ibox1{i,j});
            end
        end
        for i=1:m
            near{i}=get2(box,inear{i}); 
            inter{i}=get2(box,iinter{i});
        end
    end
    
    if modification==1
        near=box;
        inter=cell(m,1);
        iinter=cell(m,1);
    
        ibox2=get(A,r1,r2);
        for i=1:size(A,1)
            for j=1:size(A,2)
                iinter{A(i,j)}=setdiff(ibox2{i,j},A(i,j));
            end
        end

        for i=1:m
            inter{i}=get2(box,iinter{i});
        end
    end
    
    invR_svd=cell(m,1);
    invR_near=cell(m,1);
    for ibox=1:m
        invR_svd{ibox}=kernel(inter{ibox},box{ibox});
        invR_near{ibox}=kernel(box{ibox},near{ibox});
    end
end

% Obtain box indices within a radius(r)/near field (r1=1,r2=1)
function ibox=get(A,r1,r2)
    ibox=cell(size(A));
    for i=1:size(A,1)  
        for j=1:size(A,2)
            down=min([size(A,1),i+r2]);
            up=max([1,i-r2]);
            left=max([1,j-r1]);
            right=min([size(A,2),j+r1]);
            ibox{i,j}=A(up:down,left:right);
        end
    end

    % Arrange in ascend order
    for i=1:size(A,1)
        for j=1:size(A,2)
            ibox{i,j}=reshape(ibox{i,j},1,[]);
            ibox{i,j}=sort(ibox{i,j});
        end
    end  
end

% Obtain the indices of points (near-field)
function ipoint=get2(box,index)
    ipoint=box{index(1)}';
    for i=2:size(index,2)
        ipoint=[ipoint box{index(i)}'];
    end
end

% Create Z-order indices; n must be the index of 2
function A=Zorder(n)
    if n==2
        A=[1 2; 3 4];
    else
        B=Zorder(n/2);
        A=[B B+(n/2)^2; B+(n/2)^2*2 B+(n/2)^2*3];
    end
end

%% Notes
% The output variables are cell arrays and are described as follows:
% iinter - the indices of boxes in the interaction field of each box.
% near - the indices of observations in each box.
% inter - the indices of observations in the interaction field of each box.
% invR_svd - the (interaction-field) sub-matrices of invR for each box.
% invR_near - the (near-field) sub-matrices of invR for each box.