%% The local SVD-FMM
% http://dx.doi.org/10.17864/1947.329
% Please see the README file for more information including licence.

function [box]=create_box(x,nbox)
    lon=[min(x(:,1)),max(x(:,1))];
    lat=[min(x(:,2)),max(x(:,2))];
    
    box=cell(nbox*nbox,1);
    A=Zorder(nbox);
    
    intx=(lon(2)-lon(1))/nbox;
    inty=(lat(2)-lat(1))/nbox;
    
    for ix=1:size(x,1)
        i=floor((x(ix,1)-lon(1))/intx)+1;
        if i>nbox
            i=nbox;
        end
        j=floor((x(ix,2)-lat(1))/inty)+1;
        if j>nbox
            j=nbox;
        end     
        j=nbox+1-j;
        box{A(i,j)}(end+1,1)=ix;
    end
end

% Create Z-order indices; n must be the exponential of 2
function A = Zorder(n)
    if n == 2
        A = [1 2; 3 4];
    else
        B = Zorder(n/2);
        A = [B B+(n/2)^2; B+(n/2)^2*2 B+(n/2)^2*3];
    end
end

%% Notes
% The output "box" is a cell array.
% The cell index is box index with a Z-curve ordering.
% Each cell element contains the indices of observations in a box.