%% The local SVD-FMM
% http://dx.doi.org/10.17864/1947.329
% Please see the README file for more information including licence.

function x = generate_locations(res,ifplot)
    lon=[-6,6];
    lat=[54,60];
    r=6300; %km
    direction='v';
    
    d = great_circle_dist(lon,[lat(1) lat(1)],r);
    lon_num=floor(d/res);
    d = great_circle_dist([0 0],lat,r);
    lat_num=floor(d/res);

    N=lon_num*lat_num;
         
    %longitude
    x1=zeros(lat_num,lon_num);
    v1 = linspace(lon(1),lon(2),lon_num);
    
    for i = 1: lat_num
        x1(i,:) = v1;
    end
    
    %latitude
    x2=zeros(lat_num,lon_num);
    for j = 1:lon_num
        x2(:,j) = linspace(lat(1),lat(2),lat_num);
    end

    %put grid points in a vector
    x=zeros(N,2);
    if direction=='v'
        x(:,1)=reshape(x1,1,[]);
        x(:,2)=reshape(x2,1,[]);
    elseif direction=='h'
        x(:,1)=reshape(x1',1,[]);
        x(:,2)=reshape(x2',1,[]);
    end
        
    m=8;
    if ifplot==1
        figure('Renderer', 'painters', 'Position', [0 0 900 600])
        worldmap(lat,lon)
        setm(gca,'MeridianLabel','on','ParallelLabel','on','MLineLocation',(lon(2)-lon(1))/m,...
            'PLineLocation',(lat(2)-lat(1))/m,'MLabelLocation',2,'PLabelLocation',1,...
            'GLineStyle','-','GLineWidth',1,'GColor','k','FontSize',8);
        for i=1:N
            geoshow(x(i,2),x(i,1), 'DisplayType', 'Point', 'Marker', '.','Color',[0.75 0.75 0.75],...
            'MarkerEdgeColor',[0.75 0.75 0.75],'Markersize',8);
        end
        set(gcf,'Units','inches');
        screenposition = get(gcf,'Position');
        set(gcf,'PaperPosition',[0.5 0.5 screenposition(3:4)],'PaperSize',[screenposition(3:4)+1]);
        print -dpdf -painters epsFig
    end
end

function d = great_circle_dist(lon,lat,r)
    lambda = lon/180.0e0*pi;
    phi = lat/180.0e0*pi; 

    dlambda=abs(lambda(2)-lambda(1));
    dphi=abs(phi(2)-phi(1));

    angle=2.0e0*asin(sqrt( sin(dphi*0.5e0)^2 + cos(phi(1))*cos(phi(2))*sin(dlambda*0.5e0)^2));
    d = abs(r*angle);
end

%% Notes
% The output is a N x 2 matrix, where N is the number of observations.
% Each row contains the location of an observation (longitude, latitude)