%% The local SVD-FMM
% http://dx.doi.org/10.17864/1947.329
% Please see the README file for more information including licence.

%% Parameter setting
% Number of singular vectors and singular values
p=10;

% Localisation length parameter 
% (r1-latitudinal direction; r2-longitudinal direction)
% Must larger 
r1=1;
r2=1;

% Correlation function (SOAR/FOAR)
CorrelationFunction="FOAR"; 

% Observation error correlation lengthscale (km)
L=80; 

% Modificatin to the near field (0-No/1-Yes)
modification=1;

% Repetition of experiments with different vector of d
repeat=100;

%% The locations of observations
% Spatial resolution of observations (grid-length in km)
res=12;       

% Plotting the locations of observations (0-No/1-Yes) 
ifplot=0;

% Obtain the longitude and latitude of each location
x=generate_locations(res,ifplot);

%% Observation and background error covariance matrices
% Observation error standard deviation
sd=1;

% Observation error covariance matrix
[R,Cinv,distx]=corr_func(x,sd,L,CorrelationFunction);

% Background error covariance matrix
[P,Cinvb,distxb]=corr_func(x,sd*0.6,20,CorrelationFunction);

invR=inv(R);

%% The local SVD-FMM algorithm
% Divide the domain of observation locations into boxes
nbox=8; 
% The number of boxes in one dimension; the total number is 8*8
% Must be an index of 2 to allow for Z-ordering
% Must ensure that increasing nbox does not result in any empty boxes
box=create_box(x,nbox);

% Define the near/interaction/far fields
[iinter,near,inter,invR_svd,invR_near]=algorithm_fields(box,invR,r1,r2,modification);
[U,S,V,B]=algorithm_decomposition(invR_svd,box,inter,iinter,p);

rmse=0;
N=size(invR,1);

% Generate innovation vector
d_samples=generate_d(R,P,repeat,N);

for irun=1:repeat
    d=d_samples(irun,:)';
    % Reference value
    ref=invR*d;
    % Near-field and interaction-field computations
    [q_near,q_inter]=algorithm_core(invR_near,d,B,S,V,box,near,iinter);
    fmm=reordering(q_near,q_inter,size(ref),box);
    rmse=rmse+sqrt(mean((ref-fmm).^2))/repeat;
end

disp('The root-mean-squared error of the local SVD-FMM is:')
disp(rmse)